@@ -17,22 +17,21 @@ async def handle_500(request, response):
 
 
 def error_pages(overrides):
-    async def middleware(app, handler):
-        async def middleware_handler(request):
-            try:
-                response = await handler(request)
-                override = overrides.get(response.status)
-                if override is None:
-                    return response
-                else:
-                    return await override(request, response)
-            except web.HTTPException as ex:
-                override = overrides.get(ex.status)
-                if override is None:
-                    raise
-                else:
-                    return await override(request, ex)
-        return middleware_handler
+    @web.middleware
+    async def middleware(request, handler):
+        try:
+            response = await handler(request)
+            override = overrides.get(response.status)
+            if override is None:
+                return response
+            else:
+                return await override(request, response)
+        except web.HTTPException as ex:
+            override = overrides.get(ex.status)
+            if override is None:
+                raise
+            else:
+                return await override(request, ex)
     return middleware
 
 
