@@ -6,6 +6,7 @@ import io
 import os.path
 import urllib.parse
 import zlib
+from http.cookies import SimpleCookie
 from unittest import mock
 
 import pytest
@@ -13,12 +14,13 @@ from multidict import CIMultiDict, CIMultiDictProxy, istr
 from yarl import URL
 
 import aiohttp
-from aiohttp import BaseConnector, hdrs, helpers, payload
-from aiohttp.client_reqrep import ClientRequest, ClientResponse
-from aiohttp.helpers import SimpleCookie
+from aiohttp import BaseConnector, hdrs, payload
+from aiohttp.client_reqrep import (ClientRequest, ClientResponse, Fingerprint,
+                                   _merge_ssl_params)
+from aiohttp.test_utils import make_mocked_coro
 
 
-@pytest.yield_fixture
+@pytest.fixture
 def make_request(loop):
     request = None
 
@@ -37,39 +39,37 @@ def buf():
     return bytearray()
 
 
-@pytest.yield_fixture
+@pytest.fixture
+def protocol(loop):
+    protocol = mock.Mock()
+    protocol._drain_helper.return_value = loop.create_future()
+    protocol._drain_helper.return_value.set_result(None)
+    return protocol
+
+
+@pytest.fixture
 def transport(buf):
     transport = mock.Mock()
 
     def write(chunk):
         buf.extend(chunk)
 
-    @asyncio.coroutine
-    def write_eof():
+    async def write_eof():
         pass
 
     transport.write.side_effect = write
     transport.write_eof.side_effect = write_eof
+    transport.is_closing.return_value = False
 
     return transport
 
 
 @pytest.fixture
-def conn(stream):
-    return mock.Mock(writer=stream)
-
-
-@pytest.fixture
-def stream(buf, transport):
-    stream = mock.Mock()
-    stream.transport = transport
-
-    def acquire(writer):
-        writer.set_transport(transport)
-
-    stream.acquire.side_effect = acquire
-    stream.drain.return_value = ()
-    return stream
+def conn(transport, protocol):
+    return mock.Mock(
+        transport=transport,
+        protocol=protocol
+    )
 
 
 def test_method1(make_request):
@@ -99,7 +99,9 @@ def test_version_default(make_request):
 
 def test_request_info(make_request):
     req = make_request('get', 'http://python.org/')
-    assert req.request_info == (URL('http://python.org/'), 'GET', req.headers)
+    assert req.request_info == aiohttp.RequestInfo(URL('http://python.org/'),
+                                                   'GET',
+                                                   req.headers)
 
 
 def test_version_err(make_request):
@@ -143,49 +145,49 @@ def test_host_port_default_https(make_request):
     req = make_request('get', 'https://python.org/')
     assert req.host == 'python.org'
     assert req.port == 443
-    assert req.ssl
+    assert req.is_ssl()
 
 
 def test_host_port_nondefault_http(make_request):
     req = make_request('get', 'http://python.org:960/')
     assert req.host == 'python.org'
     assert req.port == 960
-    assert not req.ssl
+    assert not req.is_ssl()
 
 
 def test_host_port_nondefault_https(make_request):
     req = make_request('get', 'https://python.org:960/')
     assert req.host == 'python.org'
     assert req.port == 960
-    assert req.ssl
+    assert req.is_ssl()
 
 
 def test_host_port_default_ws(make_request):
     req = make_request('get', 'ws://python.org/')
     assert req.host == 'python.org'
     assert req.port == 80
-    assert not req.ssl
+    assert not req.is_ssl()
 
 
 def test_host_port_default_wss(make_request):
     req = make_request('get', 'wss://python.org/')
     assert req.host == 'python.org'
     assert req.port == 443
-    assert req.ssl
+    assert req.is_ssl()
 
 
 def test_host_port_nondefault_ws(make_request):
     req = make_request('get', 'ws://python.org:960/')
     assert req.host == 'python.org'
     assert req.port == 960
-    assert not req.ssl
+    assert not req.is_ssl()
 
 
 def test_host_port_nondefault_wss(make_request):
     req = make_request('get', 'wss://python.org:960/')
     assert req.host == 'python.org'
     assert req.port == 960
-    assert req.ssl
+    assert req.is_ssl()
 
 
 def test_host_port_err(make_request):
@@ -306,34 +308,33 @@ def test_ipv6_default_https_port(make_request):
     req = make_request('get', 'https://[2001:db8::1]/')
     assert req.host == '2001:db8::1'
     assert req.port == 443
-    assert req.ssl
+    assert req.is_ssl()
 
 
 def test_ipv6_nondefault_http_port(make_request):
     req = make_request('get', 'http://[2001:db8::1]:960/')
     assert req.host == '2001:db8::1'
     assert req.port == 960
-    assert not req.ssl
+    assert not req.is_ssl()
 
 
 def test_ipv6_nondefault_https_port(make_request):
     req = make_request('get', 'https://[2001:db8::1]:960/')
     assert req.host == '2001:db8::1'
     assert req.port == 960
-    assert req.ssl
+    assert req.is_ssl()
 
 
 def test_basic_auth(make_request):
     req = make_request('get', 'http://python.org',
-                       auth=aiohttp.helpers.BasicAuth('nkim', '1234'))
+                       auth=aiohttp.BasicAuth('nkim', '1234'))
     assert 'AUTHORIZATION' in req.headers
     assert 'Basic bmtpbToxMjM0' == req.headers['AUTHORIZATION']
 
 
 def test_basic_auth_utf8(make_request):
     req = make_request('get', 'http://python.org',
-                       auth=aiohttp.helpers.BasicAuth('nkim', 'секрет',
-                                                      'utf-8'))
+                       auth=aiohttp.BasicAuth('nkim', 'секрет', 'utf-8'))
     assert 'AUTHORIZATION' in req.headers
     assert 'Basic bmtpbTrRgdC10LrRgNC10YI=' == req.headers['AUTHORIZATION']
 
@@ -501,8 +502,7 @@ def test_gen_netloc_no_port(make_request):
         '012345678901234567890'
 
 
-@asyncio.coroutine
-def test_connection_header(loop, conn):
+async def test_connection_header(loop, conn):
     req = ClientRequest('get', URL('http://python.org'), loop=loop)
     req.keep_alive = mock.Mock()
     req.headers.clear()
@@ -510,128 +510,123 @@ def test_connection_header(loop, conn):
     req.keep_alive.return_value = True
     req.version = (1, 1)
     req.headers.clear()
-    req.send(conn)
+    await req.send(conn)
     assert req.headers.get('CONNECTION') is None
 
     req.version = (1, 0)
     req.headers.clear()
-    req.send(conn)
+    await req.send(conn)
     assert req.headers.get('CONNECTION') == 'keep-alive'
 
     req.keep_alive.return_value = False
     req.version = (1, 1)
     req.headers.clear()
-    req.send(conn)
+    await req.send(conn)
     assert req.headers.get('CONNECTION') == 'close'
 
 
-@asyncio.coroutine
-def test_no_content_length(loop, conn):
+async def test_no_content_length(loop, conn):
     req = ClientRequest('get', URL('http://python.org'), loop=loop)
-    resp = req.send(conn)
+    resp = await req.send(conn)
     assert req.headers.get('CONTENT-LENGTH') is None
-    yield from req.close()
+    await req.close()
     resp.close()
 
 
-@asyncio.coroutine
-def test_no_content_length_head(loop, conn):
+async def test_no_content_length_head(loop, conn):
     req = ClientRequest('head', URL('http://python.org'), loop=loop)
-    resp = req.send(conn)
+    resp = await req.send(conn)
     assert req.headers.get('CONTENT-LENGTH') is None
-    yield from req.close()
+    await req.close()
     resp.close()
 
 
-def test_content_type_auto_header_get(loop, conn):
+async def test_content_type_auto_header_get(loop, conn):
     req = ClientRequest('get', URL('http://python.org'), loop=loop)
-    resp = req.send(conn)
+    resp = await req.send(conn)
     assert 'CONTENT-TYPE' not in req.headers
     resp.close()
 
 
-def test_content_type_auto_header_form(loop, conn):
+async def test_content_type_auto_header_form(loop, conn):
     req = ClientRequest('post', URL('http://python.org'),
                         data={'hey': 'you'}, loop=loop)
-    resp = req.send(conn)
+    resp = await req.send(conn)
     assert 'application/x-www-form-urlencoded' == \
         req.headers.get('CONTENT-TYPE')
     resp.close()
 
 
-def test_content_type_auto_header_bytes(loop, conn):
+async def test_content_type_auto_header_bytes(loop, conn):
     req = ClientRequest('post', URL('http://python.org'), data=b'hey you',
                         loop=loop)
-    resp = req.send(conn)
+    resp = await req.send(conn)
     assert 'application/octet-stream' == req.headers.get('CONTENT-TYPE')
     resp.close()
 
 
-def test_content_type_skip_auto_header_bytes(loop, conn):
+async def test_content_type_skip_auto_header_bytes(loop, conn):
     req = ClientRequest('post', URL('http://python.org'), data=b'hey you',
                         skip_auto_headers={'Content-Type'},
                         loop=loop)
-    resp = req.send(conn)
+    resp = await req.send(conn)
     assert 'CONTENT-TYPE' not in req.headers
     resp.close()
 
 
-def test_content_type_skip_auto_header_form(loop, conn):
+async def test_content_type_skip_auto_header_form(loop, conn):
     req = ClientRequest('post', URL('http://python.org'),
                         data={'hey': 'you'}, loop=loop,
                         skip_auto_headers={'Content-Type'})
-    resp = req.send(conn)
+    resp = await req.send(conn)
     assert 'CONTENT-TYPE' not in req.headers
     resp.close()
 
 
-def test_content_type_auto_header_content_length_no_skip(loop, conn):
+async def test_content_type_auto_header_content_length_no_skip(loop, conn):
     req = ClientRequest('post', URL('http://python.org'),
                         data=io.BytesIO(b'hey'),
                         skip_auto_headers={'Content-Length'},
                         loop=loop)
-    resp = req.send(conn)
+    resp = await req.send(conn)
     assert req.headers.get('CONTENT-LENGTH') == '3'
     resp.close()
 
 
-def test_urlencoded_formdata_charset(loop, conn):
+async def test_urlencoded_formdata_charset(loop, conn):
     req = ClientRequest(
         'post', URL('http://python.org'),
         data=aiohttp.FormData({'hey': 'you'}, charset='koi8-r'), loop=loop)
-    req.send(conn)
+    await req.send(conn)
     assert 'application/x-www-form-urlencoded; charset=koi8-r' == \
         req.headers.get('CONTENT-TYPE')
 
 
-@asyncio.coroutine
-def test_post_data(loop, conn):
+async def test_post_data(loop, conn):
     for meth in ClientRequest.POST_METHODS:
         req = ClientRequest(
             meth, URL('http://python.org/'),
             data={'life': '42'}, loop=loop)
-        resp = req.send(conn)
+        resp = await req.send(conn)
         assert '/' == req.url.path
         assert b'life=42' == req.body._value
         assert 'application/x-www-form-urlencoded' ==\
             req.headers['CONTENT-TYPE']
-        yield from req.close()
+        await req.close()
         resp.close()
 
 
-@asyncio.coroutine
-def test_pass_falsy_data(loop):
+async def test_pass_falsy_data(loop):
     with mock.patch(
             'aiohttp.client_reqrep.ClientRequest.update_body_from_data'):
         req = ClientRequest(
             'post', URL('http://python.org/'),
             data={}, loop=loop)
         req.update_body_from_data.assert_called_once_with({})
-    yield from req.close()
+    await req.close()
 
 
-@asyncio.coroutine
-def test_pass_falsy_data_file(loop, tmpdir):
+async def test_pass_falsy_data_file(loop, tmpdir):
     testfile = tmpdir.join('tmpfile').open('w+b')
     testfile.write(b'data')
     testfile.seek(0)
@@ -642,137 +637,128 @@ def test_pass_falsy_data_file(loop, tmpdir):
         skip_auto_headers=skip,
         loop=loop)
     assert req.headers.get('CONTENT-LENGTH', None) is not None
-    yield from req.close()
+    await req.close()
 
 
 # Elasticsearch API requires to send request body with GET-requests
-@asyncio.coroutine
-def test_get_with_data(loop):
+async def test_get_with_data(loop):
     for meth in ClientRequest.GET_METHODS:
         req = ClientRequest(
             meth, URL('http://python.org/'), data={'life': '42'},
             loop=loop)
         assert '/' == req.url.path
         assert b'life=42' == req.body._value
-        yield from req.close()
+        await req.close()
 
 
-@asyncio.coroutine
-def test_bytes_data(loop, conn):
+async def test_bytes_data(loop, conn):
     for meth in ClientRequest.POST_METHODS:
         req = ClientRequest(
             meth, URL('http://python.org/'),
             data=b'binary data', loop=loop)
-        resp = req.send(conn)
+        resp = await req.send(conn)
         assert '/' == req.url.path
         assert isinstance(req.body, payload.BytesPayload)
         assert b'binary data' == req.body._value
         assert 'application/octet-stream' == req.headers['CONTENT-TYPE']
-        yield from req.close()
+        await req.close()
         resp.close()
 
 
-@asyncio.coroutine
-def test_content_encoding(loop, conn):
+async def test_content_encoding(loop, conn):
     req = ClientRequest('post', URL('http://python.org/'), data='foo',
                         compress='deflate', loop=loop)
-    with mock.patch('aiohttp.client_reqrep.PayloadWriter') as m_writer:
-        resp = req.send(conn)
+    with mock.patch('aiohttp.client_reqrep.StreamWriter') as m_writer:
+        m_writer.return_value.write_headers = make_mocked_coro()
+        resp = await req.send(conn)
     assert req.headers['TRANSFER-ENCODING'] == 'chunked'
     assert req.headers['CONTENT-ENCODING'] == 'deflate'
     m_writer.return_value\
         .enable_compression.assert_called_with('deflate')
-    yield from req.close()
+    await req.close()
     resp.close()
 
 
-@asyncio.coroutine
-def test_content_encoding_dont_set_headers_if_no_body(loop, conn):
+async def test_content_encoding_dont_set_headers_if_no_body(loop, conn):
     req = ClientRequest('post', URL('http://python.org/'),
                         compress='deflate', loop=loop)
     with mock.patch('aiohttp.client_reqrep.http'):
-        resp = req.send(conn)
+        resp = await req.send(conn)
     assert 'TRANSFER-ENCODING' not in req.headers
     assert 'CONTENT-ENCODING' not in req.headers
-    yield from req.close()
+    await req.close()
     resp.close()
 
 
-@asyncio.coroutine
-def test_content_encoding_header(loop, conn):
+async def test_content_encoding_header(loop, conn):
     req = ClientRequest(
         'post', URL('http://python.org/'), data='foo',
         headers={'Content-Encoding': 'deflate'}, loop=loop)
-    with mock.patch('aiohttp.client_reqrep.PayloadWriter') as m_writer:
-        resp = req.send(conn)
+    with mock.patch('aiohttp.client_reqrep.StreamWriter') as m_writer:
+        m_writer.return_value.write_headers = make_mocked_coro()
+        resp = await req.send(conn)
 
     assert not m_writer.return_value.enable_compression.called
     assert not m_writer.return_value.enable_chunking.called
-    yield from req.close()
+    await req.close()
     resp.close()
 
 
-@asyncio.coroutine
-def test_compress_and_content_encoding(loop, conn):
+async def test_compress_and_content_encoding(loop, conn):
     with pytest.raises(ValueError):
         ClientRequest('post', URL('http://python.org/'), data='foo',
                       headers={'content-encoding': 'deflate'},
                       compress='deflate', loop=loop)
 
 
-@asyncio.coroutine
-def test_chunked(loop, conn):
+async def test_chunked(loop, conn):
     req = ClientRequest(
         'post', URL('http://python.org/'),
         headers={'TRANSFER-ENCODING': 'gzip'}, loop=loop)
-    resp = req.send(conn)
+    resp = await req.send(conn)
     assert 'gzip' == req.headers['TRANSFER-ENCODING']
-    yield from req.close()
+    await req.close()
     resp.close()
 
 
-@asyncio.coroutine
-def test_chunked2(loop, conn):
+async def test_chunked2(loop, conn):
     req = ClientRequest(
         'post', URL('http://python.org/'),
         headers={'Transfer-encoding': 'chunked'}, loop=loop)
-    resp = req.send(conn)
+    resp = await req.send(conn)
     assert 'chunked' == req.headers['TRANSFER-ENCODING']
-    yield from req.close()
+    await req.close()
     resp.close()
 
 
-@asyncio.coroutine
-def test_chunked_explicit(loop, conn):
+async def test_chunked_explicit(loop, conn):
     req = ClientRequest(
         'post', URL('http://python.org/'), chunked=True, loop=loop)
-    with mock.patch('aiohttp.client_reqrep.PayloadWriter') as m_writer:
-        resp = req.send(conn)
+    with mock.patch('aiohttp.client_reqrep.StreamWriter') as m_writer:
+        m_writer.return_value.write_headers = make_mocked_coro()
+        resp = await req.send(conn)
 
     assert 'chunked' == req.headers['TRANSFER-ENCODING']
     m_writer.return_value.enable_chunking.assert_called_with()
-    yield from req.close()
+    await req.close()
     resp.close()
 
 
-@asyncio.coroutine
-def test_chunked_length(loop, conn):
+async def test_chunked_length(loop, conn):
     with pytest.raises(ValueError):
         ClientRequest(
             'post', URL('http://python.org/'),
             headers={'CONTENT-LENGTH': '1000'}, chunked=True, loop=loop)
 
 
-@asyncio.coroutine
-def test_chunked_transfer_encoding(loop, conn):
+async def test_chunked_transfer_encoding(loop, conn):
     with pytest.raises(ValueError):
         ClientRequest(
             'post', URL('http://python.org/'),
             headers={'TRANSFER-ENCODING': 'chunked'}, chunked=True, loop=loop)
 
 
-@asyncio.coroutine
-def test_file_upload_not_chunked(loop):
+async def test_file_upload_not_chunked(loop):
     here = os.path.dirname(__file__)
     fname = os.path.join(here, 'sample.key')
     with open(fname, 'rb') as f:
@@ -782,11 +768,10 @@ def test_file_upload_not_chunked(loop):
             loop=loop)
         assert not req.chunked
         assert req.headers['CONTENT-LENGTH'] == str(os.path.getsize(fname))
-        yield from req.close()
+        await req.close()
 
 
-@asyncio.coroutine
-def test_precompressed_data_stays_intact(loop):
+async def test_precompressed_data_stays_intact(loop):
     data = zlib.compress(b'foobar')
     req = ClientRequest(
         'post', URL('http://python.org/'),
@@ -797,11 +782,10 @@ def test_precompressed_data_stays_intact(loop):
     assert not req.compress
     assert not req.chunked
     assert req.headers['CONTENT-ENCODING'] == 'deflate'
-    yield from req.close()
+    await req.close()
 
 
-@asyncio.coroutine
-def test_file_upload_not_chunked_seek(loop):
+async def test_file_upload_not_chunked_seek(loop):
     here = os.path.dirname(__file__)
     fname = os.path.join(here, 'sample.key')
     with open(fname, 'rb') as f:
@@ -812,11 +796,10 @@ def test_file_upload_not_chunked_seek(loop):
             loop=loop)
         assert req.headers['CONTENT-LENGTH'] == \
             str(os.path.getsize(fname) - 100)
-        yield from req.close()
+        await req.close()
 
 
-@asyncio.coroutine
-def test_file_upload_force_chunked(loop):
+async def test_file_upload_force_chunked(loop):
     here = os.path.dirname(__file__)
     fname = os.path.join(here, 'sample.key')
     with open(fname, 'rb') as f:
@@ -827,53 +810,50 @@ def test_file_upload_force_chunked(loop):
             loop=loop)
         assert req.chunked
         assert 'CONTENT-LENGTH' not in req.headers
-        yield from req.close()
+        await req.close()
 
 
-def test_expect100(loop, conn):
+async def test_expect100(loop, conn):
     req = ClientRequest('get', URL('http://python.org/'),
                         expect100=True, loop=loop)
-    resp = req.send(conn)
+    resp = await req.send(conn)
     assert '100-continue' == req.headers['EXPECT']
     assert req._continue is not None
     req.terminate()
     resp.close()
 
 
-def test_expect_100_continue_header(loop, conn):
+async def test_expect_100_continue_header(loop, conn):
     req = ClientRequest('get', URL('http://python.org/'),
                         headers={'expect': '100-continue'}, loop=loop)
-    resp = req.send(conn)
+    resp = await req.send(conn)
     assert '100-continue' == req.headers['EXPECT']
     assert req._continue is not None
     req.terminate()
     resp.close()
 
 
-@asyncio.coroutine
-def test_data_stream(loop, buf, conn):
+async def test_data_stream(loop, buf, conn):
     @aiohttp.streamer
-    def gen(writer):
-        writer.write(b'binary data')
-        writer.write(b' result')
+    async def gen(writer):
+        await writer.write(b'binary data')
+        await writer.write(b' result')
 
     req = ClientRequest(
         'POST', URL('http://python.org/'), data=gen(), loop=loop)
     assert req.chunked
     assert req.headers['TRANSFER-ENCODING'] == 'chunked'
 
-    resp = req.send(conn)
-    assert helpers.isfuture(req._writer)
-    yield from resp.wait_for_close()
+    resp = await req.send(conn)
+    assert asyncio.isfuture(req._writer)
+    await resp.wait_for_close()
     assert req._writer is None
-
     assert buf.split(b'\r\n\r\n', 1)[1] == \
         b'b\r\nbinary data\r\n7\r\n result\r\n0\r\n\r\n'
-    yield from req.close()
+    await req.close()
 
 
-@asyncio.coroutine
-def test_data_file(loop, buf, conn):
+async def test_data_file(loop, buf, conn):
     req = ClientRequest(
         'POST', URL('http://python.org/'),
         data=io.BufferedReader(io.BytesIO(b'*' * 2)),
@@ -882,139 +862,131 @@ def test_data_file(loop, buf, conn):
     assert isinstance(req.body, payload.BufferedReaderPayload)
     assert req.headers['TRANSFER-ENCODING'] == 'chunked'
 
-    resp = req.send(conn)
-    assert helpers.isfuture(req._writer)
-    yield from resp.wait_for_close()
+    resp = await req.send(conn)
+    assert asyncio.isfuture(req._writer)
+    await resp.wait_for_close()
     assert req._writer is None
     assert buf.split(b'\r\n\r\n', 1)[1] == \
         b'2\r\n' + b'*' * 2 + b'\r\n0\r\n\r\n'
-    yield from req.close()
+    await req.close()
 
 
-@asyncio.coroutine
-def test_data_stream_exc(loop, conn):
-    fut = helpers.create_future(loop)
+async def test_data_stream_exc(loop, conn):
+    fut = loop.create_future()
 
     @aiohttp.streamer
-    def gen(writer):
-        writer.write(b'binary data')
-        yield from fut
+    async def gen(writer):
+        await writer.write(b'binary data')
+        await fut
 
     req = ClientRequest(
         'POST', URL('http://python.org/'), data=gen(), loop=loop)
     assert req.chunked
     assert req.headers['TRANSFER-ENCODING'] == 'chunked'
 
-    @asyncio.coroutine
-    def exc():
-        yield from asyncio.sleep(0.01, loop=loop)
+    async def throw_exc():
+        await asyncio.sleep(0.01, loop=loop)
         fut.set_exception(ValueError)
 
-    helpers.ensure_future(exc(), loop=loop)
+    loop.create_task(throw_exc())
 
-    req.send(conn)
-    yield from req._writer
+    await req.send(conn)
+    await req._writer
     # assert conn.close.called
     assert conn.protocol.set_exception.called
-    yield from req.close()
+    await req.close()
 
 
-@asyncio.coroutine
-def test_data_stream_exc_chain(loop, conn):
-    fut = helpers.create_future(loop)
+async def test_data_stream_exc_chain(loop, conn):
+    fut = loop.create_future()
 
     @aiohttp.streamer
-    def gen(writer):
-        yield from fut
+    async def gen(writer):
+        await fut
 
     req = ClientRequest('POST', URL('http://python.org/'),
                         data=gen(), loop=loop)
 
     inner_exc = ValueError()
 
-    @asyncio.coroutine
-    def exc():
-        yield from asyncio.sleep(0.01, loop=loop)
+    async def throw_exc():
+        await asyncio.sleep(0.01, loop=loop)
         fut.set_exception(inner_exc)
 
-    helpers.ensure_future(exc(), loop=loop)
+    loop.create_task(throw_exc())
 
-    req.send(conn)
-    yield from req._writer
+    await req.send(conn)
+    await req._writer
     # assert connection.close.called
     assert conn.protocol.set_exception.called
     outer_exc = conn.protocol.set_exception.call_args[0][0]
     assert isinstance(outer_exc, ValueError)
     assert inner_exc is outer_exc
     assert inner_exc is outer_exc
-    yield from req.close()
+    await req.close()
 
 
-@asyncio.coroutine
-def test_data_stream_continue(loop, buf, conn):
+async def test_data_stream_continue(loop, buf, conn):
     @aiohttp.streamer
-    def gen(writer):
-        writer.write(b'binary data')
-        writer.write(b' result')
-        yield from writer.write_eof()
+    async def gen(writer):
+        await writer.write(b'binary data')
+        await writer.write(b' result')
+        await writer.write_eof()
 
     req = ClientRequest(
         'POST', URL('http://python.org/'), data=gen(),
         expect100=True, loop=loop)
     assert req.chunked
 
-    def coro():
-        yield from asyncio.sleep(0.0001, loop=loop)
+    async def coro():
+        await asyncio.sleep(0.0001, loop=loop)
         req._continue.set_result(1)
 
-    helpers.ensure_future(coro(), loop=loop)
+    loop.create_task(coro())
 
-    resp = req.send(conn)
-    yield from req._writer
+    resp = await req.send(conn)
+    await req._writer
     assert buf.split(b'\r\n\r\n', 1)[1] == \
         b'b\r\nbinary data\r\n7\r\n result\r\n0\r\n\r\n'
-    yield from req.close()
+    await req.close()
     resp.close()
 
 
-@asyncio.coroutine
-def test_data_continue(loop, buf, conn):
+async def test_data_continue(loop, buf, conn):
     req = ClientRequest(
         'POST', URL('http://python.org/'), data=b'data',
         expect100=True, loop=loop)
 
-    def coro():
-        yield from asyncio.sleep(0.0001, loop=loop)
+    async def coro():
+        await asyncio.sleep(0.0001, loop=loop)
         req._continue.set_result(1)
 
-    helpers.ensure_future(coro(), loop=loop)
+    loop.create_task(coro())
 
-    resp = req.send(conn)
+    resp = await req.send(conn)
 
-    yield from req._writer
+    await req._writer
     assert buf.split(b'\r\n\r\n', 1)[1] == b'data'
-    yield from req.close()
+    await req.close()
     resp.close()
 
 
-@asyncio.coroutine
-def test_close(loop, buf, conn):
+async def test_close(loop, buf, conn):
     @aiohttp.streamer
-    def gen(writer):
-        yield from asyncio.sleep(0.00001, loop=loop)
-        writer.write(b'result')
+    async def gen(writer):
+        await asyncio.sleep(0.00001, loop=loop)
+        await writer.write(b'result')
 
     req = ClientRequest(
         'POST', URL('http://python.org/'), data=gen(), loop=loop)
-    resp = req.send(conn)
-    yield from req.close()
+    resp = await req.send(conn)
+    await req.close()
     assert buf.split(b'\r\n\r\n', 1)[1] == b'6\r\nresult\r\n0\r\n\r\n'
-    yield from req.close()
+    await req.close()
     resp.close()
 
 
-@asyncio.coroutine
-def test_custom_response_class(loop, conn):
+async def test_custom_response_class(loop, conn):
     class CustomResponse(ClientResponse):
         def read(self, decode=False):
             return 'customized!'
@@ -1022,31 +994,29 @@ def test_custom_response_class(loop, conn):
     req = ClientRequest(
         'GET', URL('http://python.org/'), response_class=CustomResponse,
         loop=loop)
-    resp = req.send(conn)
+    resp = await req.send(conn)
     assert 'customized!' == resp.read()
-    yield from req.close()
+    await req.close()
     resp.close()
 
 
-@asyncio.coroutine
-def test_oserror_on_write_bytes(loop, conn):
+async def test_oserror_on_write_bytes(loop, conn):
     req = ClientRequest(
         'POST', URL('http://python.org/'), loop=loop)
 
     writer = mock.Mock()
     writer.write.side_effect = OSError
 
-    yield from req.write_bytes(writer, conn)
+    await req.write_bytes(writer, conn)
 
     assert conn.protocol.set_exception.called
     exc = conn.protocol.set_exception.call_args[0][0]
     assert isinstance(exc, aiohttp.ClientOSError)
 
 
-@asyncio.coroutine
-def test_terminate(loop, conn):
+async def test_terminate(loop, conn):
     req = ClientRequest('get', URL('http://python.org'), loop=loop)
-    resp = req.send(conn)
+    resp = await req.send(conn)
     assert req._writer is not None
     writer = req._writer = mock.Mock()
 
@@ -1057,10 +1027,18 @@ def test_terminate(loop, conn):
 
 
 def test_terminate_with_closed_loop(loop, conn):
-    req = ClientRequest('get', URL('http://python.org'), loop=loop)
-    resp = req.send(conn)
-    assert req._writer is not None
-    writer = req._writer = mock.Mock()
+    req = resp = writer = None
+
+    async def go():
+        nonlocal req, resp, writer
+        req = ClientRequest('get', URL('http://python.org'))
+        resp = await req.send(conn)
+        assert req._writer is not None
+        writer = req._writer = mock.Mock()
+
+        await asyncio.sleep(0.05)
+
+    loop.run_until_complete(go())
 
     loop.close()
     req.terminate()
@@ -1077,13 +1055,12 @@ def test_terminate_without_writer(loop):
     assert req._writer is None
 
 
-@asyncio.coroutine
-def test_custom_req_rep(loop):
+async def test_custom_req_rep(loop):
     conn = None
 
     class CustomResponse(ClientResponse):
-        @asyncio.coroutine
-        def start(self, connection, read_until_eof=False):
+
+        async def start(self, connection, read_until_eof=False):
             nonlocal conn
             conn = connection
             self.status = 123
@@ -1096,19 +1073,23 @@ def test_custom_req_rep(loop):
 
     class CustomRequest(ClientRequest):
 
-        def send(self, conn):
+        async def send(self, conn):
             resp = self.response_class(self.method,
                                        self.url,
                                        writer=self._writer,
-                                       continue100=self._continue)
-            resp._post_init(self.loop, mock.Mock())
+                                       continue100=self._continue,
+                                       timer=self._timer,
+                                       request_info=self.request_info,
+                                       auto_decompress=self._auto_decompress,
+                                       traces=self._traces,
+                                       loop=self.loop,
+                                       session=self._session)
             self.response = resp
             nonlocal called
             called = True
             return resp
 
-    @asyncio.coroutine
-    def create_connection(req):
+    async def create_connection(req, traces=None):
         assert isinstance(req, CustomRequest)
         return mock.Mock()
     connector = BaseConnector(loop=loop)
@@ -1120,33 +1101,32 @@ def test_custom_req_rep(loop):
         connector=connector,
         loop=loop)
 
-    resp = yield from session.request(
+    resp = await session.request(
         'get', URL('http://example.com/path/to'))
     assert isinstance(resp, CustomResponse)
     assert called
     resp.close()
-    session.close()
+    await session.close()
     conn.close()
 
 
 def test_verify_ssl_false_with_ssl_context(loop):
-    with pytest.raises(ValueError):
-        ClientRequest('get', URL('http://python.org'), verify_ssl=False,
-                      ssl_context=mock.Mock(), loop=loop)
+    with pytest.warns(DeprecationWarning):
+        with pytest.raises(ValueError):
+            _merge_ssl_params(None, verify_ssl=False,
+                              ssl_context=mock.Mock(), fingerprint=None)
 
 
 def test_bad_fingerprint(loop):
     with pytest.raises(ValueError):
-        ClientRequest('get', URL('http://python.org'),
-                      fingerprint=b'invalid', loop=loop)
+        Fingerprint(b'invalid')
 
 
-def test_insecure_fingerprint(loop):
-    with pytest.warns(DeprecationWarning):
-        ClientRequest('get', URL('http://python.org'),
-                      fingerprint=hashlib.md5(b"foo").digest(),
-                      loop=loop)
-    with pytest.warns(DeprecationWarning):
-        ClientRequest('get', URL('http://python.org'),
-                      fingerprint=hashlib.sha1(b"foo").digest(),
-                      loop=loop)
+def test_insecure_fingerprint_md5(loop):
+    with pytest.raises(ValueError):
+        Fingerprint(hashlib.md5(b"foo").digest())
+
+
+def test_insecure_fingerprint_sha1(loop):
+    with pytest.raises(ValueError):
+        Fingerprint(hashlib.sha1(b"foo").digest())
