@@ -1,7 +1,5 @@
 """Tests for aiohttp/protocol.py"""
 
-import asyncio
-import unittest
 import zlib
 from unittest import mock
 
@@ -61,6 +59,11 @@ def response_cls(request):
     return request.param
 
 
+@pytest.fixture
+def stream():
+    return mock.Mock()
+
+
 def test_parse_headers(parser):
     text = b'''GET /test HTTP/1.1\r
 test: line\r
@@ -93,23 +96,21 @@ def test_parse(parser):
     assert msg.version == (1, 1)
 
 
-@asyncio.coroutine
-def test_parse_body(parser):
+async def test_parse_body(parser):
     text = b'GET /test HTTP/1.1\r\nContent-Length: 4\r\n\r\nbody'
     messages, upgrade, tail = parser.feed_data(text)
     assert len(messages) == 1
     _, payload = messages[0]
-    body = yield from payload.read(4)
+    body = await payload.read(4)
     assert body == b'body'
 
 
-@asyncio.coroutine
-def test_parse_body_with_CRLF(parser):
+async def test_parse_body_with_CRLF(parser):
     text = b'\r\nGET /test HTTP/1.1\r\nContent-Length: 4\r\n\r\nbody'
     messages, upgrade, tail = parser.feed_data(text)
     assert len(messages) == 1
     _, payload = messages[0]
-    body = yield from payload.read(4)
+    body = await payload.read(4)
     assert body == b'body'
 
 
@@ -254,7 +255,7 @@ def test_request_chunked(parser):
     msg, payload = messages[0]
     assert msg.chunked
     assert not upgrade
-    assert isinstance(payload, streams.FlowControlStreamReader)
+    assert isinstance(payload, streams.StreamReader)
 
 
 def test_conn_upgrade(parser):
@@ -315,7 +316,7 @@ def test_headers_connect(parser):
     messages, upgrade, tail = parser.feed_data(text)
     msg, payload = messages[0]
     assert upgrade
-    assert isinstance(payload, streams.FlowControlStreamReader)
+    assert isinstance(payload, streams.StreamReader)
 
 
 def test_headers_old_websocket_key1(parser):
@@ -519,7 +520,7 @@ def test_http_request_chunked_payload(parser):
 
     assert msg.chunked
     assert not payload.is_eof()
-    assert isinstance(payload, streams.FlowControlStreamReader)
+    assert isinstance(payload, streams.StreamReader)
 
     parser.feed_data(b'4\r\ndata\r\n4\r\nline\r\n0\r\n\r\n')
 
@@ -650,30 +651,26 @@ def test_url_parse_non_strict_mode(parser):
     assert payload.is_eof()
 
 
-class TestParsePayload(unittest.TestCase):
+class TestParsePayload:
 
-    def setUp(self):
-        self.stream = mock.Mock()
-        asyncio.set_event_loop(None)
-
-    def test_parse_eof_payload(self):
-        out = aiohttp.FlowControlDataQueue(self.stream)
+    def test_parse_eof_payload(self, stream):
+        out = aiohttp.FlowControlDataQueue(stream)
         p = HttpPayloadParser(out, readall=True)
         p.feed_data(b'data')
         p.feed_eof()
 
-        self.assertTrue(out.is_eof())
-        self.assertEqual([(bytearray(b'data'), 4)], list(out._buffer))
+        assert out.is_eof()
+        assert [(bytearray(b'data'), 4)] == list(out._buffer)
 
-    def test_parse_no_body(self):
-        out = aiohttp.FlowControlDataQueue(self.stream)
+    def test_parse_no_body(self, stream):
+        out = aiohttp.FlowControlDataQueue(stream)
         p = HttpPayloadParser(out, method='PUT')
 
-        self.assertTrue(out.is_eof())
-        self.assertTrue(p.done)
+        assert out.is_eof()
+        assert p.done
 
-    def test_parse_length_payload_eof(self):
-        out = aiohttp.FlowControlDataQueue(self.stream)
+    def test_parse_length_payload_eof(self, stream):
+        out = aiohttp.FlowControlDataQueue(stream)
 
         p = HttpPayloadParser(out, length=4)
         p.feed_data(b'da')
@@ -681,115 +678,112 @@ class TestParsePayload(unittest.TestCase):
         with pytest.raises(http_exceptions.ContentLengthError):
             p.feed_eof()
 
-    def test_parse_chunked_payload_size_error(self):
-        out = aiohttp.FlowControlDataQueue(self.stream)
+    def test_parse_chunked_payload_size_error(self, stream):
+        out = aiohttp.FlowControlDataQueue(stream)
         p = HttpPayloadParser(out, chunked=True)
-        self.assertRaises(
-            http_exceptions.TransferEncodingError, p.feed_data, b'blah\r\n')
-        self.assertIsInstance(
-            out.exception(), http_exceptions.TransferEncodingError)
+        with pytest.raises(http_exceptions.TransferEncodingError):
+            p.feed_data(b'blah\r\n')
+        assert isinstance(out.exception(),
+                          http_exceptions.TransferEncodingError)
 
-    def test_http_payload_parser_length(self):
-        out = aiohttp.FlowControlDataQueue(self.stream)
+    def test_http_payload_parser_length(self, stream):
+        out = aiohttp.FlowControlDataQueue(stream)
         p = HttpPayloadParser(out, length=2)
         eof, tail = p.feed_data(b'1245')
-        self.assertTrue(eof)
+        assert eof
 
-        self.assertEqual(b'12', b''.join(d for d, _ in out._buffer))
-        self.assertEqual(b'45', tail)
+        assert b'12' == b''.join(d for d, _ in out._buffer)
+        assert b'45' == tail
 
     _comp = zlib.compressobj(wbits=-zlib.MAX_WBITS)
     _COMPRESSED = b''.join([_comp.compress(b'data'), _comp.flush()])
 
-    def test_http_payload_parser_deflate(self):
+    def test_http_payload_parser_deflate(self, stream):
         length = len(self._COMPRESSED)
-        out = aiohttp.FlowControlDataQueue(self.stream)
+        out = aiohttp.FlowControlDataQueue(stream)
         p = HttpPayloadParser(
             out, length=length, compression='deflate')
         p.feed_data(self._COMPRESSED)
-        self.assertEqual(b'data', b''.join(d for d, _ in out._buffer))
-        self.assertTrue(out.is_eof())
+        assert b'data' == b''.join(d for d, _ in out._buffer)
+        assert out.is_eof()
 
-    def test_http_payload_parser_deflate_no_wbits(self):
+    def test_http_payload_parser_deflate_no_wbits(self, stream):
         comp = zlib.compressobj()
         COMPRESSED = b''.join([comp.compress(b'data'), comp.flush()])
 
         length = len(COMPRESSED)
-        out = aiohttp.FlowControlDataQueue(self.stream)
+        out = aiohttp.FlowControlDataQueue(stream)
         p = HttpPayloadParser(
             out, length=length, compression='deflate')
         p.feed_data(COMPRESSED)
-        self.assertEqual(b'data', b''.join(d for d, _ in out._buffer))
-        self.assertTrue(out.is_eof())
+        assert b'data' == b''.join(d for d, _ in out._buffer)
+        assert out.is_eof()
 
-    def test_http_payload_parser_length_zero(self):
-        out = aiohttp.FlowControlDataQueue(self.stream)
+    def test_http_payload_parser_length_zero(self, stream):
+        out = aiohttp.FlowControlDataQueue(stream)
         p = HttpPayloadParser(out, length=0)
-        self.assertTrue(p.done)
-        self.assertTrue(out.is_eof())
+        assert p.done
+        assert out.is_eof()
 
     @pytest.mark.skipif(brotli is None, reason="brotli is not installed")
-    def test_http_payload_brotli(self):
+    def test_http_payload_brotli(self, stream):
         compressed = brotli.compress(b'brotli data')
-        out = aiohttp.FlowControlDataQueue(self.stream)
+        out = aiohttp.FlowControlDataQueue(stream)
         p = HttpPayloadParser(
             out, length=len(compressed), compression='br')
         p.feed_data(compressed)
-        self.assertEqual(b'brotli data', b''.join(d for d, _ in out._buffer))
-        self.assertTrue(out.is_eof())
+        assert b'brotli data' == b''.join(d for d, _ in out._buffer)
+        assert out.is_eof()
 
 
-class TestDeflateBuffer(unittest.TestCase):
+class TestDeflateBuffer:
 
-    def setUp(self):
-        self.stream = mock.Mock()
-        asyncio.set_event_loop(None)
-
-    def test_feed_data(self):
-        buf = aiohttp.FlowControlDataQueue(self.stream)
+    def test_feed_data(self, stream):
+        buf = aiohttp.FlowControlDataQueue(stream)
         dbuf = DeflateBuffer(buf, 'deflate')
 
         dbuf.decompressor = mock.Mock()
         dbuf.decompressor.decompress.return_value = b'line'
 
         dbuf.feed_data(b'data', 4)
-        self.assertEqual([b'line'], list(d for d, _ in buf._buffer))
+        assert [b'line'] == list(d for d, _ in buf._buffer)
 
-    def test_feed_data_err(self):
-        buf = aiohttp.FlowControlDataQueue(self.stream)
+    def test_feed_data_err(self, stream):
+        buf = aiohttp.FlowControlDataQueue(stream)
         dbuf = DeflateBuffer(buf, 'deflate')
 
         exc = ValueError()
         dbuf.decompressor = mock.Mock()
         dbuf.decompressor.decompress.side_effect = exc
 
-        self.assertRaises(
-            http_exceptions.ContentEncodingError, dbuf.feed_data, b'data', 4)
+        with pytest.raises(http_exceptions.ContentEncodingError):
+            dbuf.feed_data(b'data', 4)
 
-    def test_feed_eof(self):
-        buf = aiohttp.FlowControlDataQueue(self.stream)
+    def test_feed_eof(self, stream):
+        buf = aiohttp.FlowControlDataQueue(stream)
         dbuf = DeflateBuffer(buf, 'deflate')
 
         dbuf.decompressor = mock.Mock()
         dbuf.decompressor.flush.return_value = b'line'
 
         dbuf.feed_eof()
-        self.assertEqual([b'line'], list(d for d, _ in buf._buffer))
-        self.assertTrue(buf._eof)
+        assert [b'line'] == list(d for d, _ in buf._buffer)
+        assert buf._eof
 
-    def test_feed_eof_err(self):
-        buf = aiohttp.FlowControlDataQueue(self.stream)
+    def test_feed_eof_err(self, stream):
+        buf = aiohttp.FlowControlDataQueue(stream)
         dbuf = DeflateBuffer(buf, 'deflate')
 
         dbuf.decompressor = mock.Mock()
         dbuf.decompressor.flush.return_value = b'line'
         dbuf.decompressor.eof = False
 
-        self.assertRaises(http_exceptions.ContentEncodingError, dbuf.feed_eof)
+        with pytest.raises(http_exceptions.ContentEncodingError):
+            dbuf.feed_eof()
 
-    def test_empty_body(self):
-        buf = aiohttp.FlowControlDataQueue(self.stream)
+    def test_empty_body(self, stream):
+        buf = aiohttp.FlowControlDataQueue(stream)
         dbuf = DeflateBuffer(buf, 'deflate')
         dbuf.feed_eof()
 
-        self.assertTrue(buf.at_eof())
+        assert buf.at_eof()
